//================================================ file = demux.c =============
//=  Demultiplexor for VIQ PPS switch simulation model                        =
//=============================================================================
//=  Contains: demux()
//=============================================================================
//=  Notes:  1) Traffic dispatch algorithm is round robin by flow             =
//=---------------------------------------------------------------------------=
//=  History: AAS (04/01/03) - Genesis                                        =    
//=============================================================================
#define EXTERNAL        // This is an external file (used in common.h)

//----- Include files ---------------------------------------------------------
#include <assert.h>     // Needed for assert() macro
#include "csim.h"       // Needed for CSIM
#include "common.h"     // Needed for common stuff
//=============================================================================

void demux(int in_id,int out_id,double org_time)
{

  int channel_num; // Internal switch plane #, to dispatch a cell 

  create("demux");

  // Drop the extra cells of an inport-outport flow
  if(Tag[in_id][out_id]>=FLOW_CELLS)
  {
    return;
  }

  // Finding the internal switch plane in round robin order (by a flow)
  channel_num  = Next_plane[in_id][out_id];
  Next_plane[in_id][out_id] = (Next_plane[in_id][out_id]+1)%K;
  assert(Next_plane[in_id][out_id]<K);

  // Send cell to an internal channel queue at demux
  chan[channel_num][in_id](in_id,out_id,channel_num,
                           Tag[in_id][out_id],org_time);
  // Increment the sequence # (by flow)
  Tag[in_id][out_id]++;
}

//=============================================================================
//==  Function for an internal input channel at demux                        ==
//=============================================================================
void channel(int in_id,int out_id,int channel_num,int tag, double org_time)
{
  create("channel");

  // Drop the cell, if Queue is full
  if(qlength(Channel_server[channel_num][in_id])==N)
  {
    Demux_loss[in_id]++;
    return;
  }

  // Queue cell on the channel queue
  reserve(Channel_server[channel_num][in_id]);
  hold(K * CELL_TIME);
  release(Channel_server[channel_num][in_id]);

  // Forward cell to inport of the selected switch plane
  in[channel_num][in_id](in_id,out_id,channel_num,tag,org_time);
  return;
}
